# coding=utf-8
from __future__ import unicode_literals, print_function, absolute_import
import traceback
import pandas as pd
import six

from bw import utils
from bw.constant import DATA_TYPE_TICK

from bw.model.fundamental import FundamentalApi

from bw.model.history import HistoryApi

from bw.retrying import retry
from bw.utils import load_to_datetime_str, standard_fields, bwsdklogger

from bw.enum import ADJUST_NONE,ADJUST_PREV,ADJUST_POST


fundamentalapi = FundamentalApi()

historyapi = HistoryApi()

pd.set_option('precision', 4)


def reset_historyapi():
    # historyapi.reset_addr()
    pass


def reset_fundamentalapi():
    fundamentalapi.reset_addr()


def condune_error(func):
    """
    调用尝试
    """

    def wrapper(*args, **kw):
        try:
            return func(*args, **kw)
        except Exception as e:
            # print('--'*20,e)
            func_name = func.__name__
            bwsdklogger.exception(e)
            dfval = kw.get('df', None)
            if dfval:  # df 参数为true
                return pd.DataFrame([])
            else:
                if func_name in {'get_previous_trading_date', 'get_next_trading_date', }:  # 这两个函数比较特殊
                    return ""
                return []

    return wrapper



def get_fundmental_set():
    
    fset = [
        {"name":"每股经营活动产生的现金流量净额","db":"SHSZADB","collections":"MEIGUJINGYINGHUODONGCHANSHENGDEXIANJINLIULIANGJINE",
"field":"MEIGUJINGYINGHUODONGCHANSHENGDEXIANJINLIULIANGJINE","date1":"10/23/2019 11:21:03","date2":"10/23/2019 11:21:12"},

{"name":"最新户均持股比例","db":"SHSZADB","collections":"ZUIXINGUDONGHUSHU",
"field":"ZUIXINHUJUNCHIGUBILI","date1":"10/23/2019 11:21:15","date2":"10/23/2019 11:21:18"},

{"name":"最新户均持股市值","db":"SHSZADB","collections":"ZUIXINGUDONGHUSHU",
"field":"ZUIXINHUJUNCHIGUSHIZHI","date1":"10/23/2019 11:21:21","date2":"10/23/2019 11:21:24"},

{"name":"上市日期","db":"SHSZADB","collections":"SHANGSHIRIQI",
"field":"SHANGSHIRIQI","date1":"10/23/2019 11:21:48","date2":"10/23/2019 11:21:51"},

{"name":"归属于母公司股东净利润同比增长率","db":"SHSZADB","collections":"GUISHUYUMUGONGSIGUDONGJINGLIRUN",
"field":"GUISHUYUMUGONGSIGUDONGJINGLIRUNZENGZHANGLV","date1":"10/23/2019 11:21:57","date2":"10/23/2019 11:21:54"},

{"name":"上市天数","db":"SHSZADB","collections":"SHANGSHITIANSHU",
"field":"SHANGSHITIANSHU","date1":"10/23/2019 11:22:01","date2":"10/23/2019 11:22:04"},

{"name":"申万三级行业","db":"SHSZADB","collections":"SHENWANHANGYE",
"field":"SHENWANSANJIHANGYE","date1":"10/23/2019 11:22:11","date2":"10/23/2019 11:22:08"},

{"name":"申万一级行业","db":"SHSZADB","collections":"SHENWANHANGYE",
"field":"SHENWANYIJIHANGYE","date1":"10/23/2019 11:22:13","date2":"10/23/2019 11:23:02"},

{"name":"申万行业","db":"SHSZADB","collections":"SHENWANHANGYE",
"field":"SHENWANHANGYE","date1":"10/23/2019 11:22:16","date2":"10/23/2019 11:23:05"},

{"name":"股票市场","db":"SHSZADB","collections":"GUPIAOSHICHANG",
"field":"GUPIAOSHICHANG","date1":"10/23/2019 11:22:18","date2":"10/23/2019 11:23:07"},

{"name":"大宗交易成交价格","db":"SHSZADB","collections":"DAZHONGJIAOYICHENGJIAOJUNXIA",
"field":"DAZHONGJIAOYICHENGJIAOJUNXIA","date1":"10/23/2019 11:22:22","date2":"10/23/2019 11:23:09"},

{"name":"流通B股","db":"SHSZADB","collections":"LIUTONGBGU",
"field":"LIUTONGBGU","date1":"10/23/2019 11:22:24","date2":"10/23/2019 11:23:12"},

{"name":"流通A股","db":"SHSZADB","collections":"LIUTONGAGU",
"field":"LIUTONGAGU","date1":"10/23/2019 11:22:27","date2":"10/23/2019 11:23:14"},

{"name":"申万二级行业","db":"SHSZADB","collections":"SHENWANHANGYE",
"field":"SHENWANERJIHANGYE","date1":"10/23/2019 11:22:30","date2":"10/23/2019 11:23:17"},

{"name":"所属概念","db":"SHSZADB","collections":"SUOSHUGAINIAN",
"field":"SUOSHUGAINIAN","date1":"10/23/2019 11:22:32","date2":"10/23/2019 11:23:21"},

{"name":"预告日期","db":"SHSZADB","collections":"YEJIYUGAORIQI",
"field":"YEJIYUGAORIQI","date1":"10/23/2019 11:22:35","date2":"10/23/2019 11:23:24"},

{"name":"业绩变脸","db":"SHSZADB","collections":"YEJIBIANLIAN",
"field":"YEJIBIANLIAN","date1":"10/23/2019 11:22:37","date2":"10/23/2019 11:23:26"},

{"name":"大宗交易日期","db":"SHSZADB","collections":"DAZONGJIAOYIRIQI",
"field":"DAZONGJIAOYIRIQI","date1":"10/23/2019 11:22:40","date2":"10/23/2019 11:23:29"},

{"name":"市盈率TTM","db":"SHSZADB","collections":"SHIYINGLVTTM",
"field":"SHIYINGLVTTM","date1":"10/23/2019 11:22:43","date2":"10/23/2019 11:23:31"},

{"name":"停牌截止时间","db":"SHSZADB","collections":"TINGPAIRIQIZHUANGTAI",
"field":"TINGPAIJIEZHISHIJIAN","date1":"10/23/2019 11:22:46","date2":"10/23/2019 11:23:38"},

{"name":"停牌开始时间","db":"SHSZADB","collections":"TINGPAIRIQIZHUANGTAI",
"field":"TINGPAIKAISHISHIJIAN","date1":"10/23/2019 11:22:48","date2":"10/23/2019 11:23:35"},

{"name":"停牌日期","db":"SHSZADB","collections":"TINGPAIRIQIZHUANGTAI",
"field":"TINGPAIRIQI","date1":"10/23/2019 11:22:54","date2":"10/23/2019 11:23:41"},

{"name":"上榜原因","db":"SHSZADB","collections":"LONGHUBANGRIQIYUANYIN",
"field":"LONGHUBANGSHANGBANYUANYIN","date1":"10/23/2019 11:22:56","date2":"10/23/2019 11:23:44"},

{"name":"上榜日期","db":"SHSZADB","collections":"LONGHUBANGRIQIYUANYIN",
"field":"LONGHUBANGSHANGBANRIQI","date1":"10/23/2019 11:22:59","date2":"10/23/2019 11:23:46"},

{"name":"存货周转天数","db":"SHSZADB","collections":"CUNHUOZHOUZHUANTIANSHU",
"field":"CUNHUOZHOUZHUANTIANSHU","date1":"11/4/2019 14:09:43","date2":"11/4/2019 14:09:43"},

{"name":"业绩预告净利润变动幅度下限","db":"SHSZADB","collections":"YEJIYUGAOJINESHUJU",
"field":"YEJIYUGAOJINGLIRUNBIANDONGFUDUXIAXIAN","date1":"11/4/2019 14:13:00","date2":"11/4/2019 14:13:00"},

{"name":"业绩预告净利润变动幅度上限","db":"SHSZADB","collections":"YEJIYUGAOJINESHUJU",
"field":"YEJIYUGAOJINGLIRUNBIANDONGFUDUSHANGXIAN","date1":"11/4/2019 14:16:26","date2":"11/4/2019 14:16:26"},

{"name":"业绩预告净利润","db":"SHSZADB","collections":"YEJIYUGAOJINESHUJU",
"field":"YEJIYUGAOJINGLIRUN","date1":"11/4/2019 14:19:04","date2":"11/4/2019 14:19:04"},

{"name":"净资产收益率ROE(摊薄|公布值)","db":"SHSZADB","collections":"JINGZICHANSHOUYILV",
"field":"TANBOJINGZICHANSHOUYILV","date1":"11/4/2019 15:28:03","date2":"11/4/2019 15:28:03"},

{"name":"每股收益EPS(扣除/基本)","db":"SHSZADB","collections":"KOUCHUHOUJIBENMEIGUSHOUYI",
"field":"KOUCHUHOUJIBENMEIGUSHOUYI","date1":"11/4/2019 16:02:06","date2":"11/4/2019 16:02:06"},

{"name":"净利润","db":"SHSZADB","collections":"JINGLIRUN",
"field":"JINGLIRUN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"拆出资金","db":"SHSZADB","collections":"CHAICHUZIJIN",
"field":"CHAICHUZIJIN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"存货","db":"SHSZADB","collections":"CUNHUO",
"field":"CUNHUO","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"非流动资产合计","db":"SHSZADB","collections":"FEILIUDONGZICHANHEJI",
"field":"FEILIUDONGZICHANHEJI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"负债合计","db":"SHSZADB","collections":"FUZHAIHEJI",
"field":"FUZHAIHEJI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"负债和股东权益合计","db":"SHSZADB","collections":"FUZHAIHEGUDONGQUANYIHEJI",
"field":"FUZHAIHEGUDONGQUANYIHEJI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"股东权益合计","db":"SHSZADB","collections":"GUDONGQUANYIHEJI",
"field":"GUDONGQUANYIHEJI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"归属于母公司股东净利润","db":"SHSZADB","collections":"GUISHUYUMUGONGSIGUDONGJINGLIRUN",
"field":"GUISHUYUMUGONGSIGUDONGJINGLIRUN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"归属于母公司股东权益合计","db":"SHSZADB","collections":"GUISHUYUMUGONGSIGUDONGQUANYIHEJI",
"field":"GUISHUYUMUGONGSIGUDONGQUANYIHEJI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"货币资金","db":"SHSZADB","collections":"HUOBIZIJIN",
"field":"HUOBIZIJIN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"交易性金融资产","db":"SHSZADB","collections":"JIAOYIXINGZIRONGZICHAN",
"field":"JIAOYIXINGZIRONGZICHAN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"结算备付金","db":"SHSZADB","collections":"JIESUANBEIFUJIN",
"field":"JIESUANBEIFUJIN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"净资产收益率","db":"SHSZADB","collections":"JINGZICHANSHOUYILV",
"field":"JIAQUANJINGZICHANSHOUYILV","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"利润总额","db":"SHSZADB","collections":"LIRUNZONGE",
"field":"LIRUNZONGE","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"利润总额同比增长率","db":"SHSZADB","collections":"LIRUNZONGE",
"field":"LIRUNZONGEZENGZHANGLV","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"流动资产合计","db":"SHSZADB","collections":"LIUDONGZICHANHEJI",
"field":"LIUDONGZICHANHEJI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"买入返售金融资产","db":"SHSZADB","collections":"MAIRUFANSHOUJINEZICHAN",
"field":"MAIRUFANSHOUJINEZICHAN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"每股净资产","db":"SHSZADB","collections":"MEIGUJINGZICHAN",
"field":"MEIGUJINGZICHAN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"每股收益","db":"SHSZADB","collections":"JIBENMEIGUSHOUYI",
"field":"JIBENMEIGUSHOUYI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"每股收益增长率","db":"SHSZADB","collections":"JIBENMEIGUSHOUYI",
"field":"JIBENMEIGUSHOUYIZENGZHANGLV","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"每股未分配利润","db":"SHSZADB","collections":"MEIGUZIBENYINGYUWEIFENPEILIRUN",
"field":"MEIGUWEIFENPEILIRUN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"每股现金流","db":"SHSZADB","collections":"MEIGUJINGYINGHUODONGCHANSHENGDEXIANJINLIULIANGJINE",
"field":"MEIGUJINGYINGHUODONGCHANSHENGDEXIANJINLIULIANGJINE","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"每股资本公积","db":"SHSZADB","collections":"MEIGUZIBENYINGYUWEIFENPEILIRUN",
"field":"MEIGUZIBENGONGJI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"内部应收款","db":"SHSZADB","collections":"NEIBUYINGSHOUKUAN",
"field":"NEIBUYINGSHOUKUAN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"其他应收款","db":"SHSZADB","collections":"QITAYINGSHOUKUAN",
"field":"QITAYINGSHOUKUAN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"上年同期净利润","db":"SHSZADB","collections":"SHANGNIANTONGQIJINGLIRUN",
"field":"SHANGNIANTONGQIJINGLIRUN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"未分配利润","db":"SHSZADB","collections":"WEIFENPEILIRUN",
"field":"WEIFENPEILIRUN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"销售毛利率","db":"SHSZADB","collections":"XIAOSHOUMAOLILV",
"field":"XIAOSHOUMAOLILV","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"业绩预告类型","db":"SHSZADB","collections":"YEJIYUGAOLEIXINGNEIRONGYUANYIN",
"field":"YEJIYUZENGYUJIAN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"一般风险准备","db":"SHSZADB","collections":"YIBANFENGXIANZHUNBEI",
"field":"YIBANFENGXIANZHUNBEI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"盈余公积","db":"SHSZADB","collections":"YINGYUGONGJI",
"field":"YINGYUGONGJI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"营业成本","db":"SHSZADB","collections":"YINGYECHENGBEN",
"field":"YINGYECHENGBEN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"营业成本同比增长率","db":"SHSZADB","collections":"YINGYECHENGBEN",
"field":"YINGYECHENGBENZENGZHANGLV","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"营业收入","db":"SHSZADB","collections":"YINGYESHOURU",
"field":"YINGYESHOURU","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"营业收入同比增长率","db":"SHSZADB","collections":"YINGYESHOURU",
"field":"YINGYESHOURUZENGZHANGLV","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"营业外收入","db":"SHSZADB","collections":"YINGYEWAISHOURU",
"field":"YINGYEWAISHOURU","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"营业外支出","db":"SHSZADB","collections":"YINGYEWAISHOURU",
"field":"YINGYEWAIZHICHU","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"营业总成本","db":"SHSZADB","collections":"YINGYEZONGCHENGBEN",
"field":"YINGYEZONGCHENGBEN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"营业总成本同比增长率","db":"SHSZADB","collections":"YINGYEZONGCHENGBEN",
"field":"YINGYEZONGCHENGBENZENGZHANGLV","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"营业总收入","db":"SHSZADB","collections":"YINGYEZONGSHOURU",
"field":"YINGYEZONGSHOURU","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"营业总收入同比增长率","db":"SHSZADB","collections":"YINGYEZONGSHOURU",
"field":"YINGYEZONGSHOURUZENGZHANGLV","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"应收保费","db":"SHSZADB","collections":"YINGSHOUBAOFEI",
"field":"YINGSHOUBAOFEI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"应收补贴款","db":"SHSZADB","collections":"YINGSHOUBUTIEKUAN",
"field":"YINGSHOUBUTIEKUAN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"应收出口退税","db":"SHSZADB","collections":"YINGSHOUCHUKOUTUISUI",
"field":"YINGSHOUCHUKOUTUISUI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"应收分保合同准备金","db":"SHSZADB","collections":"YINGSHOUFENBAOHETONGZHUNBEIJIN",
"field":"YINGSHOUFENBAOHETONGZHUNBEIJIN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"应收分保账款","db":"SHSZADB","collections":"YINGSHOUFENBAOKUAN",
"field":"YINGSHOUFENBAOKUAN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"应收股利","db":"SHSZADB","collections":"YINGSHOUGULI",
"field":"YINGSHOUGULI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"应收利息","db":"SHSZADB","collections":"YINGSHOULIXI",
"field":"YINGSHOULIXI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"应收票据","db":"SHSZADB","collections":"YINGSHOUPIAOJU",
"field":"YINGSHOUPIAOJU","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"应收账款","db":"SHSZADB","collections":"YINGSHOUZHANGKUAN",
"field":"YINGSHOUZHANGKUAN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"预付款项","db":"SHSZADB","collections":"YUFUKUANXIANG",
"field":"YUFUKUANXIANG","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"预告净利润","db":"SHSZADB","collections":"YEJIYUGAOJINGLIRUN",
"field":"YEJIYUGAOJINGLIRUN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"预告净利润变动幅度","db":"SHSZADB","collections":"YEJIYUGAOJINGLIRUNBIANDONGFUDU",
"field":"YEJIYUGAOJINGLIRUNBIANDONGFUDU","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"预告净利润上限","db":"SHSZADB","collections":"YEJIYUGAOJINGLIRUNSHANGXIAN",
"field":"YEJIYUGAOJINGLIRUNSHANGXIAN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"预告净利润下限","db":"SHSZADB","collections":"YEJIYUGAOJINGLIRUNXIAXIAN",
"field":"YEJIYUGAOJINGLIRUNXIAXIAN","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"预告每股收益","db":"SHSZADB","collections":"YEJIYUGAOMEIGUSHOUYI",
"field":"YEJIYUGAOMEIGUSHOUYI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"预告每股收益变动幅度","db":"SHSZADB","collections":"YEJIYUGAOMEIGUSHOUYI",
"field":"YEJIYUGAOMEIGUSHOUYIBIANDONGFUDU","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"资本公积","db":"SHSZADB","collections":"ZIBENGONGJI",
"field":"ZIBENGONGJI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"资产负债率","db":"SHSZADB","collections":"ZICHANFUZHAILIUDONGLV",
"field":"ZICHANFUZHAILV","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"资产总计","db":"SHSZADB","collections":"ZICHANZONGJI",
"field":"ZICHANZONGJI","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"总资产报酬率","db":"SHSZADB","collections":"ZONGZICHANBAOCHOULV",
"field":"ZONGZICHANBAOCHOULV","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"总资产净利率","db":"SHSZADB","collections":"ZONGZICHANJINGLILV",
"field":"ZONGZICHANJINGLILV","date1":"11/15/2019 15:10:23","date2":"11/15/2019 15:10:23"},

{"name":"保户储金及投资款净增加额","db":"SHSZADB","collections":"BHCJJTZKJZJE",
"field":"BHCJJTZKJZJE","date1":"12/17/2019 11:11:20","date2":"12/17/2019 11:11:20"},

{"name":"股权激励股东大会公告日期","db":"SHSZADB","collections":"GUQUANJILIGUDONGDAHUIGONGGAORI",
"field":"GUQUANJILIGUDONGDAHUIGONGGAORI","date1":"12/17/2019 11:21:12","date2":"12/17/2019 11:21:12"},

{"name":"营业利润","db":"SHSZADB","collections":"YINGYELIRUN",
"field":"YINGYELIRUN","date1":"12/17/2019 14:34:06","date2":"12/17/2019 14:34:06"},

{"name":"收取利息、手续费及佣金的现金","db":"SHSZADB","collections":"SHOUQULIXICESHI",
"field":"SHOUQULIXICESHI","date1":"12/19/2019 14:15:43","date2":"12/19/2019 14:15:43"},

{"name":"收取利息测试顿号","db":"SHSZADB","collections":"SHOUQULIXICESHI",
"field":"SHOUQULIXICESHI","date1":"12/19/2019 14:37:28","date2":"12/19/2019 14:37:28"}


    ]

    return fset



@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_fundamentals(symbols,  collection ,field,start_date, 
    end_date,limit=1000,  table="SHSZADB", df=False):
    """
    查询基本面财务数据
    """
    fields_str, fields_list = standard_fields(field, letter_upper=True)
    start_date = utils.to_datestr(start_date)
    end_date = utils.to_datestr(end_date)
    data = fundamentalapi.get_fundamentals(symbols=symbols,
                                           start_date=start_date,end_date=end_date, 
                                           field=fields_str,collection=collection,table=table, 
                                           limit=limit)

    if df:
        data = pd.DataFrame(data)
        if fields_list:
            fields_list = ['symbol', 'date', 'name','value'] + fields_list
            columns = [x for x in fields_list if x in data.columns]
            data = data[columns]

    return data


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_fundamentals_n(table, symbols, end_date, fields=None, filter=None,
                       order_by=None, count=1, df=False):
    """
    查询基本面财务数据,每个股票在end_date的前n条
    """
    fields_str, fields_list = standard_fields(fields, letter_upper=True)
    data = fundamentalapi.get_fundamentals_n(table=table, symbols=symbols,
                                             end_date=end_date,
                                             fields=fields_str,
                                             filter=filter, order_by=order_by,
                                             count=count)

    if df:
        data = pd.DataFrame(data)
        if fields_list:
            fields_list = ['symbol', 'pub_date', 'end_date'] + fields_list
            columns = [field for field in fields_list if field in data.columns]
            data = data[columns]

    return data


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_instruments(symbols=None, exchanges=None, sec_types=None, names=None,
                    skip_suspended=True, skip_st=True, fields=None, df=False):
    """
    查询最新交易标的信息,有基本数据及最新日频数据
    """
    fields_str, fields_list = standard_fields(fields, letter_upper=False)
    data = fundamentalapi.get_instruments(symbols, exchanges, sec_types, names,
                                          skip_suspended, skip_st, fields_str)

    if df:
        data = pd.DataFrame(data)
        if fields_list:
            columns = [field for field in fields_list if field in data.columns]
            data = data[columns]

    return data


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_history_instruments(symbols, fields=None, start_date=None,
                            end_date=None, df=False):
    """
    返回指定的symbols的标的日指标数据
    """
    fields_str, fields_list = standard_fields(fields, letter_upper=False)

    data = fundamentalapi.get_history_instruments(symbols, fields_str,
                                                  start_date, end_date)

    if df:
        data = pd.DataFrame(data)
        if fields_list:
            columns = [field for field in fields_list if field in data.columns]
            data = data[columns]

    return data


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_instrumentinfos(symbols=None, exchanges=None, sec_types=None,
                        names=None, fields=None, df=False):
    """
    查询交易标的基本信息
    如果没有数据的话,返回空列表. 有的话, 返回list[dict]这样的列表. 其中 listed_date, delisted_date 为 datetime 类型
    @:param fields: 可以是 'symbol, sec_type' 这样的字符串, 也可以是 ['symbol', 'sec_type'] 这样的字符list
    """
    fields_str, fields_list = standard_fields(fields, letter_upper=False)
    data = fundamentalapi.get_instrumentinfos(symbols, exchanges, sec_types,
                                              names, fields)

    if df:
        data = pd.DataFrame(data)
        if fields_list:
            columns = [field for field in fields_list if field in data.columns]
            data = data[columns]

    return data


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_constituents(index, fields=None, df=False):
    """
    查询指数最新成分股
    返回的list每项是个字典,包含的key值有:
    symbol 股票symbol
    name 名字
    """
    fields_str, fields_list = standard_fields(fields, letter_upper=False)
    data = fundamentalapi.get_constituents(index, fields, df)
    if df:
        data = pd.DataFrame(data)
        if fields_list:
            columns = [field for field in fields_list if field in data.columns]
            data = data[columns]

    return data


# @condune_error
# @retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
# def get_history_constituents(index, start_date=None, end_date=None):
#     """
#     查询指数历史成分股
#     返回的list每项是个字典,包含的key值有:
#     trade_date: 交易日期(datetime类型)
#     constituents: 一个字典. 每个股票的sybol做为key值, weight做为value值
#     """
#     return fundamentalapi.get_history_constituents(index, start_date, end_date)


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_plates(code):
    """
    查询板块股票列表
    """
    return fundamentalapi.get_plates(code)


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_industry(code):
    """
    查询行业股票列表
    """
    return fundamentalapi.get_industry(code)


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_concept(code):
    """
    查询概念股票列表
    """

    return fundamentalapi.get_concept(code)


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_trading_dates(exchange, start_date, end_date):
    """
    查询交易日列表
    如果指定的市场不存在, 返回空列表. 有值的话,返回 yyyy-mm-dd 格式的列表
    """
    return fundamentalapi.get_trading_dates(exchange, start_date, end_date)


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_previous_trading_date(exchange, date):
    """
    返回指定日期的上一个交易日
    @:param exchange: 交易市场
    @:param date: 指定日期, 可以是datetime.date 或者 datetime.datetime 类型. 或者是 yyyy-mm-dd 或 yyyymmdd 格式的字符串
    @:return 返回下一交易日, 为 yyyy-mm-dd 格式的字符串, 如果不存在则返回None
    """
    return fundamentalapi.get_previous_trading_date(exchange, date)


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_next_trading_date(exchange, date):
    """
    返回指定日期的下一个交易日
    @:param exchange: 交易市场
    @:param date: 指定日期, 可以是datetime.date 或者 datetime.datetime 类型. 或者是 yyyy-mm-dd 或 yyyymmdd 格式的字符串
    @:return 返回下一交易日, 为 yyyy-mm-dd 格式的字符串, 如果不存在则返回None
    """
    return fundamentalapi.get_next_trading_date(exchange, date)


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_dividend(symbol, start_date, end_date=None, df=False):
    """
    查询分红送配
    """
    data = fundamentalapi.get_dividend(symbol, start_date, end_date)

    if df:
        data = pd.DataFrame(data)

    return data


@condune_error
@retry(pre_func=reset_fundamentalapi, stop_max_attempt_number=5)
def get_continuous_contracts(csymbol, start_date=None, end_date=None):
    """
    获取连续合约
    """
    return fundamentalapi.get_continuous_contracts(csymbol, start_date,
                                                   end_date)


@condune_error
@retry(pre_func=reset_historyapi, stop_max_attempt_number=5)
def history(symbol, frequency, start_time, end_time, fields=None,
            skip_suspended=True, fill_missing=None, 
            adjust=1, df=False):
    """
    查询历史行情
    """
    start_time = load_to_datetime_str(start_time)
    end_time = load_to_datetime_str(end_time)
    # adjust_end_time = load_to_datetime_str(adjust_end_time)
    symbol = symbol.strip()
    frequency = frequency.strip()
    # print("&"*20,start_time,end_time,symbol,frequency)
    if frequency == DATA_TYPE_TICK:
        return historyapi.get_history_ticks(symbols=symbol,
                                            start_time=start_time,
                                            end_time=end_time, fields=fields,
                                            skip_suspended=skip_suspended,
                                            
                                            adjust=adjust,
                                            # adjust_end_time=adjust_end_time,
                                            df=df)

    else:
        return historyapi.get_history_bars(symbol=symbol, frequency=frequency,
                                           start_time=start_time,
                                           end_time=end_time, fields=fields,
                                          
                                           adjust=adjust,
                                        #    adjust_end_time=adjust_end_time,
                                           df=df)


@condune_error
@retry(pre_func=reset_historyapi, stop_max_attempt_number=5)
def history_n(symbol, frequency, count, end_time=None, fields=None,
              skip_suspended=True, fill_missing=None, adjust=ADJUST_PREV,
              adjust_end_time='', df=False):
    """
    查询历史行情
    """
    end_time = load_to_datetime_str(end_time)
    # adjust_end_time = load_to_datetime_str(adjust_end_time)
        
    try:
        symbol = symbol.strip()
        frequency = frequency.strip()

        if frequency == DATA_TYPE_TICK:
            return historyapi.get_history_n_ticks(symbol=symbol, count=count,
                                                end_time=end_time, fields=fields,
                                                
                                                fill_missing=fill_missing,
                                                adjust=adjust,
                                                df=df)

        else:
            return historyapi.get_history_n_bars(symbol=symbol, frequency=frequency,
                                                count=count, end_time=end_time,
                                                fields=fields,
                                                fill_missing=fill_missing,
                                                adjust=adjust,
                                                df=df)

        pass
    except Exception as identifier:
        print('Error occured :',identifier)
        # exit(0)
        pass
                                             
                                        



# coding=utf-8
from __future__ import print_function, absolute_import, unicode_literals

import getopt
import sys

# 基本api
from bw import __version__
# from bw.csdk.cpp_sdk import py_bwi_set_version
from bw.model.storage import Context
from bw.enum import *
from .basic import (
    set_token,set_srv_cache, get_version, subscribe, unsubscribe,
    current, get_strerror, schedule, run,  log, set_serv_addr, stop,set_cache)
# 数据查询api
from .query import (
    history_n, history, get_fundamentals,
    # get_dividend, 
    get_fundmental_set,
    # get_continuous_contracts,
     get_next_trading_date,
    get_previous_trading_date, get_trading_dates, get_concept, get_industry, get_constituents,
    #get_history_constituents,get_history_instruments, get_instrumentinfos, get_instruments,
get_plates
#, get_fundamentals_n
)
# 交易api
from .trade import (
   order_target_volume,
    order_volume, 
    order_percent, 
    order_value,
    order_target_value,
    order_target_percent,
    #  order_batch,
    order_cancel_all,
    order_cancel, order_close_all,
    get_orders,get_positions, get_unfinished_orders, get_execution_reports)

__all__ = [
    'order_target_volume', 
    'order_target_value',
     'order_target_percent',
    'order_volume', 
    'order_percent', 
    'order_value', 
    # 'order_batch',

    'order_cancel_all', 'order_cancel', 'order_close_all',
    'get_orders', 'get_positions','get_unfinished_orders','get_execution_reports',

    'set_token','set_srv_cache', 'get_version', 'subscribe', 'unsubscribe',
    'current', 'get_strerror', 'schedule', 'run','set_cache',
    'log', 'stop',

    'history_n', 'history',
     'get_fundamentals', 
     'get_fundmental_set',
    # 'get_dividend',
    # 'get_continuous_contracts', 
    'get_next_trading_date',
    'get_previous_trading_date', 'get_trading_dates', 
    'get_plates', 
    'get_concept',
    'get_industry',
    'get_constituents', 
    # 'get_history_constituents',
    # 'get_history_instruments', 'get_instrumentinfos', 'get_fundamentals_n',
    # 'get_instruments', 
    'set_serv_addr',

    'Context',

    'ExecType_Unknown',
    'ExecType_New',
    'ExecType_DoneForDay',
    'ExecType_Canceled',
    'ExecType_PendingCancel',
    'ExecType_Stopped',
    'ExecType_Rejected',
    'ExecType_Suspended',
    'ExecType_PendingNew',
    'ExecType_Calculated',
    'ExecType_Expired',
    'ExecType_Restated',
    'ExecType_PendingReplace',
    'ExecType_Trade',
    'ExecType_TradeCorrect',
    'ExecType_TradeCancel',
    'ExecType_OrderStatus',
    'ExecType_CancelRejected',
    'OrderStatus_Unknown',
    'OrderStatus_New',
    'OrderStatus_PartiallyFilled',
    'OrderStatus_Filled',
    'OrderStatus_DoneForDay',
    'OrderStatus_Canceled',
    'OrderStatus_PendingCancel',
    'OrderStatus_Stopped',
    'OrderStatus_Rejected',
    'OrderStatus_Suspended',
    'OrderStatus_PendingNew',
    'OrderStatus_Calculated',
    'OrderStatus_Expired',
    'OrderStatus_AcceptedForBidding',
    'OrderStatus_PendingReplace',
    'OrderRejectReason_Unknown',
    'OrderRejectReason_RiskRuleCheckFailed',
    'OrderRejectReason_NoEnoughCash',
    'OrderRejectReason_NoEnoughPosition',
    'OrderRejectReason_IllegalAccountId',
    'OrderRejectReason_IllegalStrategyId',
    'OrderRejectReason_IllegalSymbol',
    'OrderRejectReason_IllegalVolume',
    'OrderRejectReason_IllegalPrice',
    'OrderRejectReason_AccountDisabled',
    'OrderRejectReason_AccountDisconnected',
    'OrderRejectReason_AccountLoggedout',
    'OrderRejectReason_NotInTradingSession',
    'OrderRejectReason_OrderTypeNotSupported',
    'OrderRejectReason_Throttle',
    'CancelOrderRejectReason_OrderFinalized',
    'CancelOrderRejectReason_UnknownOrder',
    'CancelOrderRejectReason_BrokerOption',
    'CancelOrderRejectReason_AlreadyInPendingCancel',
    'OrderSide_Unknown',
    'OrderSide_Buy',
    'OrderSide_Sell',
    'OrderType_Unknown',
    'OrderType_Limit',
    'OrderType_Market',
    'OrderType_Stop',
    'OrderDuration_Unknown',
    'OrderDuration_FAK',
    'OrderDuration_FOK',
    'OrderDuration_GFD',
    'OrderDuration_GFS',
    'OrderDuration_GTD',
    'OrderDuration_GTC',
    'OrderDuration_GFA',
    'OrderQualifier_Unknown',
    'OrderQualifier_BOC',
    'OrderQualifier_BOP',
    'OrderQualifier_B5TC',
    'OrderQualifier_B5TL',
    'OrderStyle_Unknown',
    'OrderStyle_Volume',
    'OrderStyle_Value',
    'OrderStyle_Percent',
    'OrderStyle_TargetVolume',
    'OrderStyle_TargetValue',
    'OrderStyle_TargetPercent',
    'PositionSide_Unknown',
    'PositionSide_Long',
    'PositionSide_Short',
    'PositionEffect_Unknown',
    'PositionEffect_Open',
    'PositionEffect_Close',
    'PositionEffect_CloseToday',
    'PositionEffect_CloseYesterday',
    'CashPositionChangeReason_Unknown',
    'CashPositionChangeReason_Trade',
    'CashPositionChangeReason_Inout',
    'MODE_LIVE',
    'MODE_BACKTEST',
    'ADJUST_NONE',
    'ADJUST_PREV',
    'ADJUST_POST',
    
    'SEC_TYPE_STOCK',
    'SEC_TYPE_BOND',
    'SEC_TYPE_FUND',
    'SEC_TYPE_WARRANT',
    'SEC_TYPE_INDEX',
    'SEC_TYPE_FUTURES',
    'SEC_TYPE_STOCK_KC',
    'SEC_TYPE_PLATE',
    'SEC_TYPE_CONCEPT',
]

try:
    if sys.version_info.major < 3:
        __all__ = [str(item) for item in __all__]
    ver_info = sys.version_info
    sdk_lang = "python{}.{}".format(ver_info.major, ver_info.minor)
    sdk_version = __version__.__version__

    # py_bwi_set_version(sdk_version, sdk_lang)

    command_argv = sys.argv[1:]
    options, args = getopt.getopt(command_argv, None,
                                  ['strategy_id=', 'filename=',
                                   'mode=','usr=','pwd=' ,'token=', 'apitoken=',
                                   'backtest_start_time=',
                                   'backtest_end_time=',
                                   'backtest_initial_cash=',
                                   'backtest_transaction_ratio=',
                                   'backtest_commission_ratio=',
                                   'backtest_slippage_ratio=',
                                   'backtest_adjust=',
                                   'backtest_check_cache=',
                                   'serv_addr=',
                                   'port=',  # 用于pycharm的python console时会额外传入port参数
                                   ])

    for option, value in options:
        if option == '--serv_addr' and value:
            set_serv_addr(value)
            break
except BaseException as e:
    pass



from typing import Text, Dict


# # nipasbc
# class GetCurrentTicksReq(object):
#     symbols = ...  # type: Text
#     fields = ...  # type: Text

#     def __init__(self, symbols: Text, fields: Text): ...


# # nipasbc
# class GetHistoryTicksReq(object):
#     symbols = ...  # type: Text
#     start_time = ...  # type: Text
#     end_time = ...  # type: Text
#     fields = ...  # type: Text
#     skip_suspended = ...  # type: bool
#     fill_missing = ...  # type: Text
#     adjust = ...  # type: int
#     adjust_end_time = ...  # type: Text

#     def __init__(self, symbols: Text = '', start_time: Text = '', end_time: Text = '', fields: Text = '',
#                  skip_suspended: bool = None, fill_missing: Text = '', adjust: int = None,
#                  adjust_end_time: Text = ''): ...


# # nipasbc
# class GetHistoryBarsReq(object):
#     symbols = ...  # type: Text
#     frequency = ...  # type: Text
#     start_time = ...  # type: Text
#     end_time = ...  # type: Text
#     fields = ...  # type: Text
#     skip_suspended = ...  # type: bool
#     fill_missing = ...  # type: Text
#     adjust = ...  # type: int
#     adjust_end_time = ...  # type: Text

#     def __init__(self, symbols: Text = '', frequency: Text = '', start_time: Text = '',
#                  end_time: Text = '', fields: Text = '', skip_suspended: bool = None,
#                  fill_missing: Text = '', adjust: int = None, adjust_end_time: Text = '', ): ...


# # nipasbc
# class GetHistoryTicksNReq(object):
#     symbol = ...  # type: Text
#     count = ...  # type: int
#     end_time = ...  # type: Text
#     fields = ...  # type: Text
#     skip_suspended = ...  # type: bool
#     fill_missing = ...  # type: Text
#     adjust = ...  # type: int
#     adjust_end_time = ...  # type: Text

#     def __init__(self, symbol: Text = '', count: int = None, end_time: Text = '',
#                  fields: Text = '', skip_suspended: bool = None, fill_missing: Text = '',
#                  adjust: int = None, adjust_end_time: Text = None): ...


# # nipasbc
# class GetHistoryBarsNReq(object):
#     symbol = ...  # type: Text
#     frequency = ...  # type: Text
#     count = ...  # type: int
#     end_time = ...  # type: Text
#     fields = ...  # type: Text
#     skip_suspended = ...  # type: bool
#     fill_missing = ...  # type: Text
#     adjust = ...  # type: int
#     adjust_end_time = ...  # type: Text

#     def __init__(self, symbol: Text = '', frequency: Text = '', count: int = None,
#                  end_time: Text = '', fields: Text = '', skip_suspended: bool = None,
#                  fill_missing: Text = '', adjust: int = None, adjust_end_time: Text = '',
#                  ): ...


# # nipasbc
# class GetBenchmarkReturnReq(object):
#     symbol = ...  # type: Text
#     frequency = ...  # type: Text
#     start_time = ...  # type: Text
#     end_time = ...  # type: Text
#     adjust = ...  # type: int
#     adjust_end_time = ...  # type: Text

#     def __init__(self, symbol: Text = '',
#                  frequency: Text = '',
#                  start_time: Text = '',
#                  end_time: Text = '',
#                  adjust: int = None,
#                  adjust_end_time: Text = '',
#                  ): ...


# # nipasbc
# class GetBenchmarkReturnRsp(object):
#     class BenchmarkReturn(object):
#         ratio = ...  # type: float
#         created_at = ...  # type: int

#         def __init__(self, ratio: float = None, end_date: created_at = None): ...

#     data = ...  # type: List[GetBenchmarkReturnRsp]

#     # def __init__(self, data: List[GetBenchmarkReturnRsp] = None): ...



# coding=utf-8
from typing import Text, Dict

from bw.yy.common import ConnectionStatus, ConnectionAddress,EnumTypeWrapper

from bw.csdk.cpp_sdk import CBar,CQuote,CTick,COrder,CExecRpt,CCash,CPosition,CAccount,CAccountStatus,CParameter,CIndicator

from bw.yy.data import *


ExecType_Unknown = 0
ExecType_New = 1                      #已报
ExecType_DoneForDay = 4               #
ExecType_Canceled = 5                 #已撤销
ExecType_PendingCancel = 6            #待撤销
ExecType_Stopped = 7                  #
ExecType_Rejected = 8                 #已拒绝
ExecType_Suspended = 9                #挂起
ExecType_PendingNew = 10              #待报
ExecType_Calculated = 11              #
ExecType_Expired = 12                 #过期
ExecType_Restated = 13                #
ExecType_PendingReplace = 14          #
ExecType_Trade = 15                   #成交
ExecType_TradeCorrect = 16            #
ExecType_TradeCancel = 17             #
ExecType_OrderStatus = 18             #委托状态
ExecType_CancelRejected = 19          #撤单被拒绝


OrderStatus_Unknown = 0
OrderStatus_New = 1                   #已报
OrderStatus_PartiallyFilled = 2       #部成
OrderStatus_Filled = 3                #已成
OrderStatus_DoneForDay = 4            #
OrderStatus_Canceled = 5              #已撤
OrderStatus_PendingCancel = 6         #待撤
OrderStatus_Stopped = 7               #
OrderStatus_Rejected = 8              #已拒绝
OrderStatus_Suspended = 9             #挂起
OrderStatus_PendingNew = 10           #待报
OrderStatus_Calculated = 11           #
OrderStatus_Expired = 12              #已过期
OrderStatus_AcceptedForBidding = 13   #
OrderStatus_PendingReplace = 14       #

OrderRejectReason_Unknown = 0                           #未知原因
OrderRejectReason_RiskRuleCheckFailed = 1               #不符合风控规则
OrderRejectReason_NoEnoughCash = 2                      #资金不足
OrderRejectReason_NoEnoughPosition = 3                  #仓位不足
OrderRejectReason_IllegalAccountId = 4                  #非法账户ID
OrderRejectReason_IllegalStrategyId = 5                 #非法策略ID
OrderRejectReason_IllegalSymbol = 6                     #非法交易代码
OrderRejectReason_IllegalVolume = 7                     #非法委托量
OrderRejectReason_IllegalPrice = 8                      #非法委托价
OrderRejectReason_AccountDisabled = 10                  #交易账号被禁止交易
OrderRejectReason_AccountDisconnected = 11              #交易账号未连接
OrderRejectReason_AccountLoggedout = 12                 #交易账号未登录
OrderRejectReason_NotInTradingSession = 13              #非交易时段
OrderRejectReason_OrderTypeNotSupported = 14            #委托类型不支持
OrderRejectReason_Throttle = 15                         #流控限制
OrderRejectReason_SymbolSusppended = 16                 #交易代码停牌
OrderRejectReason_Internal = 999                        #内部错误

CancelOrderRejectReason_OrderFinalized = 101            #委托已完成
CancelOrderRejectReason_UnknownOrder = 102              #未知委托
CancelOrderRejectReason_BrokerOption = 103              #柜台设置
CancelOrderRejectReason_AlreadyInPendingCancel = 104    #委托撤销中

OrderSide_Unknown = 0
OrderSide_Buy     = 1    #买入
OrderSide_Sell    = 2    #卖出


OrderType_Unknown = 0
OrderType_Limit   = 1    #限价委托
OrderType_Market  = 2    #市价委托
OrderType_Stop    = 3    #止损止盈委托

OrderDuration_Unknown = 0
OrderDuration_FAK     = 1  #即时成交剩余撤销(fill and kill)
OrderDuration_FOK     = 2  #即时全额成交或撤销(fill or kill)
OrderDuration_GFD     = 3  #当日有效(good for day)
OrderDuration_GFS     = 4  #本节有效(good for section)
OrderDuration_GTD     = 5  #指定日期前有效(goodl till date)
OrderDuration_GTC     = 6  #撤销前有效(good till cancel)
OrderDuration_GFA     = 7  #集合竞价前有效(good for auction)

OrderQualifier_Unknown = 0
OrderQualifier_BOC     = 1  #对方最优价格(best of counterparty)
OrderQualifier_BOP     = 2  #己方最优价格(best of party)
OrderQualifier_B5TC    = 3  #最优五档剩余撤销(best 5 then cancel)
OrderQualifier_B5TL    = 4  #最优五档剩余转限价(best 5 then limit)

OrderStyle_Unknown = 0
OrderStyle_Volume = 1
OrderStyle_Value = 2
OrderStyle_Percent = 3
OrderStyle_TargetVolume = 4
OrderStyle_TargetValue = 5
OrderStyle_TargetPercent = 6

PositionSide_Unknown  = 0
PositionSide_Long     = 1   #多方向
PositionSide_Short    = 2   #空方向

PositionEffect_Unknown        = 0
PositionEffect_Open           = 1     #开仓
PositionEffect_Close          = 2     #平仓,具体语义取决于对应的交易所
PositionEffect_CloseToday     = 3     #平今仓
PositionEffect_CloseYesterday = 4     #平昨仓

CashPositionChangeReason_Unknown = 0
CashPositionChangeReason_Trade   = 1  #交易
CashPositionChangeReason_Inout   = 2  #出入金/出入持仓


# nipasbc
class ExecType(EnumTypeWrapper):
    pass


# nipasbc
class OrderStatus(EnumTypeWrapper):
    pass


# nipasbc
class OrderRejectReason(EnumTypeWrapper):
    pass


# nipasbc
class OrderSide(EnumTypeWrapper):
    pass


# nipasbc
class OrderType(EnumTypeWrapper):
    pass


# nipasbc
class OrderDuration(EnumTypeWrapper):
    pass


# nipasbc
class OrderQualifier(EnumTypeWrapper):
    pass


# nipasbc
class OrderStyle(EnumTypeWrapper):
    pass


# nipasbc
class PositionSide(EnumTypeWrapper):
    pass


# nipasbc
class PositionEffect(EnumTypeWrapper):
    pass


# nipasbc
class CashPositionChangeReason(EnumTypeWrapper):
    pass


#  CBar,CQuote,CTick,COrder,CExecRpt,CCash,CPosition,CAccount,CAccountStatus,CParameter,CIndicator

# 委托定义
# nipasbc
class Order(object):
    # # 策略ID
    # strategy_id = ... # type: Text
    # # 账号ID
    # account_id = ... # type: Text
    # # 账户登录名
    # account_name = ... # type: Text

    # # 委托客户端ID
    # cl_ord_id = ... # type: Text
    # # 委托柜台ID
    # order_id = ... # type: Text
    # # 委托交易所ID
    # ex_ord_id = ... # type: Text

    # # symbol
    # symbol = ... # type: Text
    # # 买卖方向，取值参考enum OrderSide
    # side = ... # type: int
    # # 开平标志，取值参考enum PositionEffect
    # position_effect = ... # type: int
    # # 持仓方向，取值参考enum PositionSide
    # position_side = ... # type: int

    # # 委托类型，取值参考enum OrderType
    # order_type = ... # type: int
    # # 委托时间属性，取值参考enum OrderDuration
    # order_duration = ... # type: int
    # # 委托成交属性，取值参考enum OrderQualifier
    # order_qualifier = ... # type: int
    # # 委托来源，取值参考enum OrderSrc
    # order_src = ... # type: int

    # # 委托状态，取值参考enum OrderStatus
    # status = ... # type: int
    # # 委托拒绝原因，取值参考enum OrderRejectReason
    # ord_rej_reason = ... # type: int
    # # 委托拒绝原因描述
    # ord_rej_reason_detail = ... # type: int

    # # 委托价格
    # price = ... # type: float
    # # 委托止损/止盈触发价格
    # stop_price = ... # type: float

    # # 委托风格，取值参考 enum OrderStyle
    # order_style = ... # type: int
    # # 委托量
    # volume = ... # type: int
    # # 委托额
    # value = ... # type: float
    # # 委托百分比
    # percent = ... # type: float
    # # 委托目标量
    # target_volume = ... # type: int
    # # 委托目标额
    # target_value = ... # type: float
    # # 委托目标百分比
    # target_percent = ... # type: float

    # # 已成量
    # filled_volume = ... # type: int
    # # 已成均价
    # filled_vwap = ... # type: float
    # # 已成金额
    # filled_amount = ... # type: float
    # # 已成手续费
    # filled_commission = ... # type: float

    # # 委托创建时间
    # created_at = ... # type: int
    # # 委托更新时间
    # updated_at = ... # type: int
    
    def __init__(self,):
        pass

    # def __init__(self,ord):
    #     self.strategy_id =  ord.strategy_id #///策略ID  strategy_id #///策略ID  
    #     self.account_id = ord.account_id #///账号ID   account_id #///账号ID   
    #     self.account_name = ord.account_name #///账户登录account_name #///账户登录
    #     self.cl_ord_id = ord.cl_ord_id #///委托客户端cl_ord_id #///委托客户端
    #     self.order_id = ord.order_id #///委托柜台ID order_id #///委托柜台ID 
    #     self.ex_ord_id = ord.ex_ord_id #///委托交易所ex_ord_id #///委托交易所
    #     self.symbol = ord.symbol #///symbol       symbol #///symbol       
    #     self.side = ord.side #
    #     self.position_effect = ord.position_effect #
    #     self.position_side = ord.position_side #
    #     self.order_type = ord.order_type #
    #     self.order_duration = ord.order_duration #
    #     self.order_qualifier = ord.order_qualifier #
    #     self.order_src = ord.order_src #
    #     self.status = ord.status #
    #     self.ord_rej_reason = ord.ord_rej_reason #
    #     self.ord_rej_reason_detail = ord.ord_rej_reason_detail #
    #     self.price = ord.price #
    #     self.stop_price = ord.stop_price #
    #     self.order_style = ord.order_style #
    #     self.volume = ord.volume #
    #     self.value = ord.value #
    #     self.percent = ord.percent #
    #     self.target_volume = ord.target_volume #
    #     self.target_value = ord.target_value #
    #     self.target_percent = ord.target_percent #
    #     self.filled_volume = ord.filled_volume #
    #     self.filled_vwap = ord.filled_vwap #
    #     self.filled_amount = ord.filled_amount #
    #     self.filled_commission = ord.filled_commission #
    #     self.created_at = ord.created_at #
    #     self.updated_at = ord.updated_at #
    #     pass

    def FromFmt(self,ord):
        self.strategy_id =  ord.strategy_id #///策略ID  strategy_id #///策略ID  
        self.account_id = ord.account_id #///账号ID   account_id #///账号ID   
        self.account_name = ord.account_name #///账户登录account_name #///账户登录
        self.cl_ord_id = ord.cl_ord_id #///委托客户端cl_ord_id #///委托客户端
        self.order_id = ord.order_id #///委托柜台ID order_id #///委托柜台ID 
        self.ex_ord_id = ord.ex_ord_id #///委托交易所ex_ord_id #///委托交易所
        self.symbol = ord.symbol #///symbol       symbol #///symbol       
        self.side = ord.side #
        self.position_effect = ord.position_effect #
        self.position_side = ord.position_side #
        self.order_type = ord.order_type #
        self.order_duration = ord.order_duration #
        self.order_qualifier = ord.order_qualifier #
        self.order_src = ord.order_src #
        self.status = ord.status #
        self.ord_rej_reason = ord.ord_rej_reason #
        self.ord_rej_reason_detail = ord.ord_rej_reason_detail #
        self.price = ord.price #
        self.stop_price = ord.stop_price #
        self.order_style = ord.order_style #
        self.volume = ord.volume #
        self.value = ord.value #
        self.percent = ord.percent #
        self.target_volume = ord.target_volume #
        self.target_value = ord.target_value #
        self.target_percent = ord.target_percent #
        self.filled_volume = ord.filled_volume #
        self.filled_vwap = ord.filled_vwap #
        self.filled_amount = ord.filled_amount #
        self.filled_commission = ord.filled_commission #
        self.created_at = Timestamp(ord.created_at) #
        self.updated_at = Timestamp(ord.updated_at) #
        pass

# # 委托集合
# # nipasbc
# class Orders(object):
#     data = ... # type: List[Order]
#     is_combined = ... # type: int
#     properties = ... # type: Dict[Text, Text]


# 委托执行回报定义
# nipasbc
class ExecRpt(object):
    # # 策略ID
    # strategy_id = ... # type: Text
    # # 账号ID
    # account_id = ... # type: Text
    # # 账户登录名
    # account_name = ... # type: Text

    # # 委托客户端ID
    # cl_ord_id = ... # type: Text
    # # 委托柜台ID
    # order_id = ... # type: Text
    # # 委托回报ID
    # exec_id = ... # type: Text

    # symbol = ... # type: Text

    # # 开平标志，取值参考enum PositionEffect
    # position_effect = ... # type: int
    # # 买卖方向，取值参考enum OrderSide
    # side = ... # type: int
    # # 委托拒绝原因，取值参考enum OrderRejectReason
    # ord_rej_reason = ... # type: int
    # # 委托拒绝原因描述
    # ord_rej_reason_detail = ... # type: Text
    # # 执行回报类型, 取值参考enum ExecType
    # exec_type = ... # type: int

    # # 委托成交价格
    # price = ... # type: float
    # # 委托成交量
    # volume = ... # type: int
    # # 委托成交金额
    # amount = ... # type: float
    # # 委托成交手续费
    # commission = ... # type: float

    # const = ... # type: float

    # # 回报创建时间
    # created_at = ... # type: int

    def FromFmt(self,exerpt):
        self.strategy_id=exerpt.strategy_id#                  //策略ID                                                                                                        
        self.account_id=exerpt.account_id#                   //账号ID                                                       
        self.account_name=exerpt.account_name#               //账户登录名                                                                                                    
        self.cl_ord_id=exerpt.cl_ord_id#                    //委托客户端ID                                                                                                  
        self.order_id=exerpt.order_id#                     //委托柜台ID                                                                                                    
        self.exec_id=exerpt.exec_id#                      //委托回报ID                                                                                                    
        self.symbol=exerpt.symbol#                   //symbol                                                                                                        
        self.position_effect=exerpt.position_effect#                      //开平标志，取值参考enum PositionEffect                                                                         
        self.side=exerpt.side#                                 //买卖方向，取值参考enum OrderSide                                                                              
        self.ord_rej_reason=exerpt.ord_rej_reason#                       //委托拒绝原因，取值参考enum OrderRejectReason                                                                  
        self.ord_rej_reason_detail=exerpt.ord_rej_reason_detail#      //委托拒绝原因描述                                                                                              
        self.exec_type=exerpt.exec_type#                            //执行回报类型, 取值参考enum ExecType                                                                           
        self.price=exerpt.price#                                //委托成交价格                                                                                                  
        self.volume=exerpt.volume#                               //委托成交量                                                                                                    
        self.amount=exerpt.amount#                               //委托成交金额                                                                                                  
        self.commission=exerpt.commission#                           //委托成交手续费                                                                                                
        self.cost=exerpt.cost#                                 //委托成交成本金额  
        # self.created_at=exerpt.created_at#                           //回报创建时间
        self.created_at = Timestamp(exerpt.created_at) #
       
        pass




# 成交回报集合
# nipasbc
class ExecRpts(object):
    data = ... # type: List[ExecRpt]


# 资金定义
# nipasbc

class Cash(object):
    # account_id=...#                        //账号ID               
    # account_name=...#                    //账户登录名                                                                   
    # currency=...#                                  //币种                                                                         
    # nav=...#                                        //净值(const cum_inout + cum_pnl + fpnl - cum_commission)                            
    # pnl=...#                                        //净收益(const nav-cum_inout)                                                        
    # fpnl=...#                                       //浮动盈亏(const sum(each position fpnl))                                            
    # frozen=...#                                     //持仓占用资金                                                                 
    # order_frozen=...#                               //挂单冻结资金                                                                 
    # available=...#                                  //可用资金      
    # balance=...#                                    //资金余额                                                                     
    # cum_inout=...#                                  //累计出入金                                                                   
    # cum_trade=...#                                  //累计交易额                                                                   
    # cum_pnl=...#                                    //累计平仓收益(const 没扣除手续费)                                                   
    # cum_commission=...#                             //累计手续费                                                                   
    # last_trade=...#                                 //上一次交易额                                                                 
    # last_pnl=...#                                   //上一次收益                                                                   
    # last_commission=...#                            //上一次手续费                                                                 
    # last_inout=...#                                 //上一次出入金                                                                 
    # change_reason=...#                              //资金变更原因，取值参考enum CashPositionChangeReason                          
    # change_event_id=...#                    //触发资金变更事件的ID     
    # created_at=...#                                 //资金初始时间
    # updated_at=...#                                 //资金变更时间
    def __init__(self,c):
        self.account_id=c.account_id#                        //账号ID               
        self.account_name=c.account_name#                    //账户登录名                                                                   
        self.currency=c.currency#                                  //币种                                                                         
        self.nav=c.nav#                                        //净值(const cum_inout + cum_pnl + fpnl - cum_commission)                            
        self.pnl=c.pnl#                                        //净收益(const nav-cum_inout)                                                        
        self.fpnl=c.fpnl#                                       //浮动盈亏(const sum(each position fpnl))                                            
        self.frozen=c.frozen#                                     //持仓占用资金                                                                 
        self.order_frozen=c.order_frozen#                               //挂单冻结资金                                                                 
        self.available=c.available#                                  //可用资金      
        self.balance=c.balance#                                    //资金余额                                                                     
        self.cum_inout=c.cum_inout#                                  //累计出入金                                                                   
        self.cum_trade=c.cum_trade#                                  //累计交易额                                                                   
        self.cum_pnl=c.cum_pnl#                                    //累计平仓收益(const 没扣除手续费)                                                   
        self.cum_commission=c.cum_commission#                             //累计手续费                                                                   
        self.last_trade=c.last_trade#                                 //上一次交易额                                                                 
        self.last_pnl=c.last_pnl#                                   //上一次收益                                                                   
        self.last_commission=c.last_commission#                            //上一次手续费                                                                 
        self.last_inout=c.last_inout#                                 //上一次出入金                                                                 
        self.change_reason=c.change_reason#                              //资金变更原因，取值参考enum CashPositionChangeReason                          
        self.change_event_id=c.change_event_id#                    //触发资金变更事件的ID     
        # self.created_at=c.created_at#                                 //资金初始时间
        # self.updated_at=c.updated_at#                                 //资金变更时间
        
        self.created_at = Timestamp(c.created_at) #
        self.updated_at = Timestamp(c.updated_at) #
        pass
    def FromFmt(self,c):
        self.account_id=c.account_id#                        //账号ID               
        self.account_name=c.account_name#                    //账户登录名                                                                   
        self.currency=c.currency#                                  //币种                                                                         
        self.nav=c.nav#                                        //净值(const cum_inout + cum_pnl + fpnl - cum_commission)                            
        self.pnl=c.pnl#                                        //净收益(const nav-cum_inout)                                                        
        self.fpnl=c.fpnl#                                       //浮动盈亏(const sum(each position fpnl))                                            
        self.frozen=c.frozen#                                     //持仓占用资金                                                                 
        self.order_frozen=c.order_frozen#                               //挂单冻结资金                                                                 
        self.available=c.available#                                  //可用资金      
        self.balance=c.balance#                                    //资金余额                                                                     
        self.cum_inout=c.cum_inout#                                  //累计出入金                                                                   
        self.cum_trade=c.cum_trade#                                  //累计交易额                                                                   
        self.cum_pnl=c.cum_pnl#                                    //累计平仓收益(const 没扣除手续费)                                                   
        self.cum_commission=c.cum_commission#                             //累计手续费                                                                   
        self.last_trade=c.last_trade#                                 //上一次交易额                                                                 
        self.last_pnl=c.last_pnl#                                   //上一次收益                                                                   
        self.last_commission=c.last_commission#                            //上一次手续费                                                                 
        self.last_inout=c.last_inout#                                 //上一次出入金                                                                 
        self.change_reason=c.change_reason#                              //资金变更原因，取值参考enum CashPositionChangeReason                          
        self.change_event_id=c.change_event_id#                    //触发资金变更事件的ID     
        # self.created_at=c.created_at#                                 //资金初始时间
        # self.updated_at=c.updated_at#                                 //资金变更时间
        
        self.created_at = Timestamp(c.created_at) #
        self.updated_at = Timestamp(c.updated_at) #


# 资金集合
# nipasbc
class Cashes(object):
    data = ... # type: List[Cash]


# 持仓定义
# nipasbc
class Position(object):
    # # 账号ID
    # account_id = ... # type: Text
    # # 账户登录名
    # account_name = ... # type: Text

    # symbol = ... # type: Text
    # # 持仓方向，取值参考enum PositionSide
    # side = ... # type: int
    # # 总持仓量 昨持仓量(volume-volume_today)
    # volume = ... # type: int
    # # 今日持仓量
    # volume_today = ... # type: int
    # # 持仓均价
    # vwap = ... # type: float
    # # 持仓额(volume*vwap*multiplier)
    # amount = ... # type: float

    # # 当前行情价格
    # price = ... # type: float
    # # 持仓浮动盈亏((price-vwap)*volume*multiplier)
    # fpnl = ... # type: float
    # # 持仓成本(vwap*volume*multiplier*margin_ratio)
    # cost = ... # type: float
    # # 挂单冻结仓位
    # order_frozen = ... # type: int
    # # 挂单冻结今仓仓位
    # order_frozen_today = ... # type: int
    # # 可平总仓位(volume-order_frozen) 可平昨仓位(available-available_today)
    # available = ... # type: int
    # # 可平今仓位(volume_today-order_frozen_today)
    # available_today = ... # type: int

    # # 上一次成交价
    # last_price = ... # type: float
    # # 上一次成交量
    # last_volume = ... # type: int
    # # 上一次出入持仓量
    # last_inout = ... # type: int
    # # 仓位变更原因，取值参考enum CashPositionChangeReason
    # change_reason = ... # type: int
    # # 触发资金变更事件的ID
    # change_event_id = ... # type: Text

    # 持仓区间有分红配送
    has_dividend = ... # type: int

    # 建仓时间
    created_at = ... # type: int
    # 仓位变更时间
    updated_at = ... # type: int
    def __init__(self,pos):
        self.account_id=pos.account_id#              //账号ID                                       
        self.account_name=pos.account_name#          //账户登录名                                                                                       
        self.symbol=pos.symbol#              //symbol                                       
        self.side=pos.side#                            //持仓方向，取值参考enum PositionSide          
        self.volume=pos.volume#                          //总持仓量=pos.//总持仓量# 昨持仓量(const volume-volume_today)                                                          
        self.volume_today=pos.volume_today#                    //今日持仓量                                                                                       
        self.vwap=pos.vwap#                            //持仓均价                                                                                         
        self.amount=pos.amount#                          //持仓额(const volume*vwap*multiplier)                                                                   
        self.price=pos.price#                           //当前行情价格                                                                                     
        self.fpnl=pos.fpnl#                            //持仓浮动盈亏(const (price-vwap)*volume*multiplier)                                                     
        self.cost=pos.cost#                            //持仓成本(const vwap*volume*multiplier*margin_ratio)                                                    
        self.order_frozen=pos.order_frozen#                    //挂单冻结仓位                                                                                     
        self.order_frozen_today=pos.order_frozen_today#              //挂单冻结今仓仓位                                                                                 
        self.available=pos.available#                       //可平总仓位(const volume-order_frozen)=pos.//可平总仓位(const volume-order_frozen)# 可平昨仓位(const available-available_today)                           
        self.available_today=pos.available_today#                 //可平今仓位(const volume_today-order_frozen_today)                                                      
        self.last_price=pos.last_price#                      //上一次成交价                                                                                     
        self.last_volume=pos.last_volume#                     //上一次成交量                                                                                     
        self.last_inout=pos.last_inout#                      //上一次出入持仓量                                                                                 
        self.change_reason=pos.change_reason#                   //仓位变更原因，取值参考enum CashPositionChangeReason                                              
        self.change_event_id=pos.change_event_id#         //触发资金变更事件的ID                                                                             
        self.has_dividend=pos.has_dividend#                    //持仓区间有分红配送   
        # self.created_at=pos.created_at#                      //建仓时间
        # self.updated_at=pos.updated_at#                      //仓位变更时间
        
        self.created_at = Timestamp(pos.created_at) #
        self.updated_at = Timestamp(pos.updated_at) #
        pass
    def FromFmt(self,pos):
        self.account_id=pos.account_id#              //账号ID                                       
        self.account_name=pos.account_name#          //账户登录名                                                                                       
        self.symbol=pos.symbol#              //symbol                                       
        self.side=pos.side#                            //持仓方向，取值参考enum PositionSide          
        self.volume=pos.volume#                          //总持仓量=pos.//总持仓量# 昨持仓量(const volume-volume_today)                                                          
        self.volume_today=pos.volume_today#                    //今日持仓量                                                                                       
        self.vwap=pos.vwap#                            //持仓均价                                                                                         
        self.amount=pos.amount#                          //持仓额(const volume*vwap*multiplier)                                                                   
        self.price=pos.price#                           //当前行情价格                                                                                     
        self.fpnl=pos.fpnl#                            //持仓浮动盈亏(const (price-vwap)*volume*multiplier)                                                     
        self.cost=pos.cost#                            //持仓成本(const vwap*volume*multiplier*margin_ratio)                                                    
        self.order_frozen=pos.order_frozen#                    //挂单冻结仓位                                                                                     
        self.order_frozen_today=pos.order_frozen_today#              //挂单冻结今仓仓位                                                                                 
        self.available=pos.available#                       //可平总仓位(const volume-order_frozen)=pos.//可平总仓位(const volume-order_frozen)# 可平昨仓位(const available-available_today)                           
        self.available_today=pos.available_today#                 //可平今仓位(const volume_today-order_frozen_today)                                                      
        self.last_price=pos.last_price#                      //上一次成交价                                                                                     
        self.last_volume=pos.last_volume#                     //上一次成交量                                                                                     
        self.last_inout=pos.last_inout#                      //上一次出入持仓量                                                                                 
        self.change_reason=pos.change_reason#                   //仓位变更原因，取值参考enum CashPositionChangeReason                                              
        self.change_event_id=pos.change_event_id#         //触发资金变更事件的ID                                                                             
        self.has_dividend=pos.has_dividend#                    //持仓区间有分红配送   
        # self.created_at=pos.created_at#                      //建仓时间
        # self.updated_at=pos.updated_at#                      //仓位变更时间
        self.created_at = Timestamp(pos.created_at) #
        self.updated_at = Timestamp(pos.updated_at) #
        pass


# 持仓集合
# nipasbc
class Positions(object):
    data = ... # type: List[Position]


# nipasbc
class Account(object):
    # account_id = ... # type: Text
    # account_name = ... # type: Text
    # title = ... # type: Text
    # intro = ... # type: Text
    # comment = ... # type: Text

    # created_at = ... # type: int
    # updated_at = ... # type: int
    def __init__(self,act):
        self.account_id = act .account_id # type: Text
        self.account_name = act.account_id # type: Text
        self.title = act.account_id # type: Text
        self.intro = act.account_id # type: Text
        self.comment = act.account_id # type: Text

        self.created_at = act.account_id # type: int
        self.updated_at = act.account_id # type: int


# nipasbc
class Accounts(object):
    data = ... # type: List[Account]


# nipasbc
class AccountStatus(object):
    account_id = ... # type: Text
    account_name = ... # type: Text
    status = ... # type: ConnectionStatus


# nipasbc
class AccountStatuses(object):
    data = ... # type: List[AccountStatus]




#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: bwsetup.py
# Author: Leon
# Mail: skyrecca@163.com
# Created Time:  2019-8-12 16:16:16
#############################################

from setuptools import setup, find_packages            #这个包没有的可以pip一下
import glob
import os

print ( find_packages('bw'),)
print("-"*50)

dirlst2 = glob.glob(r".\bw\helper\*.*")
dirfmtls2 = [ i.replace('\\','/').replace('bw/helper/','')  for i in dirlst2]

print(dirfmtls2)
dirlst = glob.glob(r".\bw\helper\*\*.*")
dirfmtls = [ i.replace('\\','/').replace('bw/helper/','')  for i in dirlst]

dirfmtls.extend(dirfmtls2)
# print(dirfmtls)
print('-'*50)

setup(
    name = "bwtest",      #这里是pip项目发布的名称
    version = "1.5.86",  #版本号，数值大的会优先被pip
#  package_dir={'':'bw'},
    packages = find_packages(),

    keywords = ["pip", "bw","financial analyze"],
    description = "A fast backtest framework",
    long_description = "only support python 3.7 windows 64bit ",
    license = "Apache 2.0",
    python_requires='>=3.7.0',
    
    # include_package_data=True,
    package_data={
        # '':'*.*',
        'bw.helper':dirfmtls,
        'bw.csdk': ['./*.dll','./*.pyd','./*.so','./*.exe'], 
        },

    exclude_package_data ={
        '':['setup.py']},

    url = "http://www.gushi.com/",     
    author = "Leon",
    author_email = "skyrecca@163.com",


    platforms = "windows",
    install_requires = ["pandas>=0.24","six","arrow"]          #这个项目需要的第三方库
)



# coding=utf-8
from __future__ import print_function

import glob
import os
import shutil
from distutils.core import setup
from distutils.extension import Extension
from distutils.util import get_platform

from Cython.Distutils import build_ext

remove_files = ['cpp_sdk.cpp', 'cpp_sdk.pyd',
                'cpp_sdk.cp36-win32.pyd', 'cpp_sdk.cp36-win_amd64.pyd',
                'cpp_sdk.cp37-win32.pyd', 'cpp_sdk.cp37-win_amd64.pyd',
                'libbw3.so', 'bwsdk.dll']
for f in remove_files:
    if os.path.exists(f):
        os.remove(f)

platform = get_platform()
dlllibdir = './lib/{}'.format(platform)
ext_modules = [Extension("cpp_sdk",
                         ["cpp_sdk.pyx"],
                         language='c++',
                         include_dirs=['./include'],
                         library_dirs=[dlllibdir],
                         libraries=['bw3-c'],  # so文件的名字去掉前面的lib字样
                         extra_link_args=["-Wl,-rpath", "-Wl,$ORIGIN"],)
               ]

setup(
    name='cpp_sdk',
    cmdclass={'build_ext': build_ext},
    ext_modules=ext_modules,
)

dllext = '*.dll'
if platform.startswith('linux'):
    dllext = '*.so'
dllfiles = glob.iglob(os.path.join('./lib/{}/'.format(platform), dllext))
for f in dllfiles:
    if os.path.isfile(f):
        shutil.copy2(f, './{}'.format(os.path.split(f)[1]))



# coding=utf-8
"""
回调任务分发
"""
from __future__ import unicode_literals, print_function, absolute_import

import collections
import datetime
import time

import six
from typing import Any, Dict, Text, List, Set, NoReturn

from bw.constant import CALLBACK_TYPE_TICK, CALLBACK_TYPE_BAR, \
    CALLBACK_TYPE_SCHEDULE, CALLBACK_TYPE_EXECRPT, \
    CALLBACK_TYPE_ORDER, CALLBACK_TYPE_INDICATOR, CALLBACK_TYPE_CASH, \
    CALLBACK_TYPE_POSITION, CALLBACK_TYPE_PARAMETERS, CALLBACK_TYPE_ERROR, \
    CALLBACK_TYPE_TIMER, CALLBACK_TYPE_BACKTEST_FINISH, CALLBACK_TYPE_STOP, \
    CALLBACK_TYPE_TRADE_CONNECTED, CALLBACK_TYPE_DATA_CONNECTED, \
    CALLBACK_TYPE_ACCOUNTSTATUS, CALLBACK_TYPE_DATA_DISCONNECTED, CALLBACK_TYPE_TRADE_DISCONNECTED, \
    CALLBACK_TYPE_INIT,BarLikeDict2, ObjectLikeDict

from bw.csdk.cpp_sdk import py_bwi_now_plus

from bw.enum import MODE_BACKTEST, MODE_LIVE
from bw.model import DictLikeExecRpt, DictLikeOrder, DictLikeIndicator, DictLikeParameter, DictLikeAccountStatus, \
    DictLikeConnectionStatus, DictLikeError
from bw.model.storage import Context, context, BarWaitgroupInfo, BarSubInfo

from bw.yy.data import Tick, Bar, Quote
from bw.yy.account import ExecRpt, Order, Cash, Position, AccountStatus
from bw.yy.performance import Indicator
# from bw.yy.rtconf import Parameters

from bw.utils import bwsdklogger, beijing_tzinfo, timestamp2bj_datetime,timestamp2datetime,\
    DictLikeObject,ObjectLikeDict


def init_callback(data):
    context.bar_data_set.clear()  # 清空之前的
    if context.init_fun is not None:
        context.init_fun(context)


def tick_callback_new(data):
    tick = Tick(data)
    tick.FromFmt(data)
    if len(tick.quotes) < 5:
        for _ in range(len(tick.quotes), 5):
            zero_val = {'bid_p': 0, 'bid_v': 0, 'ask_p': 0, 'ask_v': 0}
            tick.quotes.append(zero_val)

    ticknew = {
        'quotes': tick.quotes,
        'symbol': tick.symbol,
        'created_at': timestamp2bj_datetime(tick.created_at),
        'price': tick.price,
        'open': tick.open,
        'high': tick.high,
        'low': tick.low,
        'cum_volume': tick.cum_volume,
        'cum_amount': tick.cum_amount,
        'cum_position': tick.cum_position,
        'last_amount': tick.last_amount,
        'last_volume': tick.last_volume,
        'trade_type': tick.trade_type,
        'receive_local_time': time.time(),  # 收到时的本地时间秒数
    }
    ticknn = ObjectLikeDict(ticknew)
    # print(ticknn)
    tick_callback(ticknn)


def tick_callback(data):
    tick = data  # type: ObjectLikeDict
    symbol = tick['symbol']
    # print(symbol)
    if symbol not in context.tick_sub_symbols:
        bwsdklogger.debug("tick data symbol=%s 不在订阅列表里, 跳过不处理", symbol)
        return

    if symbol not in context.tick_data_cache:
        bwsdklogger.debug("tick data's symbol= %s 在context.tick_data_cache key不存在, 加个key, deque长度为:%d", symbol,
                          context.max_tick_data_count)
        context.tick_data_cache[symbol] = collections.deque(maxlen=context.max_tick_data_count)

    context.tick_data_cache[symbol].appendleft(tick)
    if context.on_tick_fun is not None:
        context.on_tick_fun(context, tick)


# 回测模式下前一个eob的时间
pre_eob_in_backtest = None  # type: datetime.datetime
# 回测模式且wait_group下的bar集合. 以 frequency 作为 key
bars_in_waitgroup_backtest = collections.defaultdict(list)  # type: Dict[Text, List[BarLikeDict2]]
# 实时模式且wait_group下的bar集合  以 frequency 作为 一级key, eob做为二级key
bars_in_waitgroup_live = dict()  # type:  Dict[Text, Dict[datetime.datetime, List[BarLikeDict2]]]


def bar_callback_new(data):
    # print('---------------------------bar _call back new')
    bar = Bar(data)
    bar.FromFmt(data)
    barnew = {
        'symbol': bar.symbol,
        'eob': timestamp2bj_datetime(bar.eob),
        'bob': timestamp2bj_datetime(bar.bob),
        'open': float(bar.open),
        'close': float(bar.close),
        'high': float(bar.high),
        'low': float(bar.low),
        'volume': bar.volume,
        'amount': bar.amount,
        'pre_close': bar.pre_close,
        'position': bar.position,
        'frequency': bar.frequency,
        'receive_local_time': time.time(),  # 收到时的本地时间秒数
    }
    # print(barnew)
    # t = 
    barnn = ObjectLikeDict(barnew)
    # print('&'*28,barnn['symbol'],barnn.data)
    bar_callback(barnn)
    pass


def bar_callback(databar):
    # print('-------------------------------bar-call_back')
    bar =databar
    # print(bar)
    global pre_eob_in_backtest, bars_in_waitgroup_backtest, bars_in_waitgroup_live
    symbol, frequency = bar['symbol'], bar['frequency']  # type: Text, Text
    # print(symbol,frequency)
    if BarSubInfo(symbol, frequency) not in context.bar_sub_infos:
        bwsdklogger.debug("bar data symbol=%s frequency=%s 不在订阅列表里, 跳过不处理", symbol, frequency)
        # print('--------------',symbol,frequency)
        # print('juran')
        return
    
    k = symbol + "_" + frequency

    # wait_group = True的情况下, 就先不要放入, 不然第一个symbol得到的数据跟别的symbol的数据对不齐
    if not context.has_wait_group:
        context.bar_data_cache[k].appendleft(bar)

    if context.on_bar_fun is None:
        return

    # 没有wait_group的情况, 那就直接发了
    if not context.has_wait_group:
        context.on_bar_fun(context, [bar, ])
        return

    # wait_group = True, 但是股票不在waitgroup的列表里时, 直接发了.
    # 在调用完 on_bar_fun 后, 在把数据放入到bar_data_cache里
    barwaitgroupinfo = context.bar_waitgroup_frequency2Info.get(frequency, BarWaitgroupInfo(frequency, 0))
    if not barwaitgroupinfo.is_symbol_in(symbol):
        bwsdklogger.debug('wait_group = True, 但是股票不在waitgroup的列表里时, 直接发了, symbol=%s, frequency=%s', symbol, frequency)
        context.on_bar_fun(context, [bar, ])
        context._add_bar2bar_data_cache(k, bar)
        return

    eob = bar['eob']  # type: datetime.datetime

    if context.mode == MODE_BACKTEST:  # 处理回测模式下, wait_group = True
        # 在回测模式下, 数据都是按顺序组织好的, 所以可以认为到下一个时间点时, 就把所有的数据统一放出来就好了
        if pre_eob_in_backtest is None:
            context._temporary_now = context.now #datetime.datetime.fromtimestamp(context.now/1000).replace(tzinfo=beijing_tzinfo)
            pre_eob_in_backtest = eob
            bars_in_waitgroup_backtest[frequency].append(bar)
            context._add_bar2bar_data_cache(k, bar)
            return

        if pre_eob_in_backtest == eob:
            bars_in_waitgroup_backtest[frequency].append(bar)
            context._add_bar2bar_data_cache(k, bar)
            return

        if pre_eob_in_backtest < eob:  # 说明是下一个时间点了
            for bs in six.itervalues(bars_in_waitgroup_backtest):
                context.on_bar_fun(context, bs)

            context._add_bar2bar_data_cache(k, bar)

            pre_eob_in_backtest = eob
            context._temporary_now = context.now #  datetime.datetime.fromtimestamp(context.now/1000).replace(tzinfo=beijing_tzinfo) 
            #datetime.datetime.fromtimestamp(context.now).replace(tzinfo=beijing_tzinfo)
            bars_in_waitgroup_backtest.clear()
            bars_in_waitgroup_backtest[frequency].append(bar)
            return

        return

    if context.mode == MODE_LIVE:  # 处理实时模式下, wait_group = True
        
        bwsdklogger.warning("实时模式下暂时不支持,数据并没有运行")
        if frequency not in bars_in_waitgroup_live:
            bars_in_waitgroup_live[frequency] = dict()

        # 以eob做为key值, bar做为value值. 二级dict
        eob_bar_dict = bars_in_waitgroup_live[frequency]  # type: Dict[datetime.datetime, List[BarLikeDict2]]
        if eob not in eob_bar_dict:
            eob_bar_dict[eob] = [bar]
        else:
            eob_bar_dict[eob].append(bar)

        # 检查一下是否全部都到了. 到了的话触发一下
        if len(barwaitgroupinfo) == len(eob_bar_dict[eob]):
            bwsdklogger.debug("实时模式下, wait_group的bar都到齐了, 触发on_bar. eob=%s", eob)
            context.bar_data_cache[k].appendleft(bar)
            context.on_bar_fun(context, eob_bar_dict[eob])
            del eob_bar_dict[eob]
        else:
            context.bar_data_cache[k].appendleft(bar)

        return


def schedule_callback(data):
    # python 3 传过来的是bytes 类型， 转成str
    if isinstance(data, bytes):
        data = bytes.decode(data)
    # print('-'*100,data)
    schedule_func = context.inside_schedules.get(data)
    if not schedule_func:
        # print('schedule_callback---------none----call')
        return
    schedule_func(context)


def _or_not_none(a, b):
    # type: (Any, Any) -> bool
    """
    两个中有一个不为None
    """
    return a is not None or b is not None


def excerpt_callback(data):
    """data indicator"""
    if context.on_execution_report_fun is not None:
        excerpt = ExecRpt()
        excerpt.FromFmt(data)
        context.on_execution_report_fun(context, excerpt)


def order_callback(data):
    if context.on_order_status_fun is not None:
        order = Order()
        order.FromFmt(data)
        context.on_order_status_fun(context, order)


def indicator_callback(data):
    if context.on_backtest_finished_fun is not None:
        indicator = Indicator()
        indicator.FromFmt(data)
        context.on_backtest_finished_fun(context, indicator)
        return


def cash_callback(data):
    cash = Cash(data)
    account_id = cash['account_id']
    accounts = context.accounts
    accounts[account_id].cash = cash


def position_callback(data):
    position = Position()
    position.FromFmt(data)
    symbol = position['symbol']
    side = position['side']
    account_id = position['account_id']
    accounts = context.accounts
    position_key = '{}.{}'.format(symbol, side)
    accounts[account_id].inside_positions[position_key] = position

    if not position.get('volume'):
        if accounts[account_id].inside_positions.get(position_key):
            return accounts[account_id].inside_positions.pop(position_key)


def parameters_callback(data):
    if context.on_parameter_fun is not None:
        context.on_parameter_fun(context, data)


def default_err_callback(ctx, code, info):
    # type: (Context, Text, Text) -> NoReturn
    if code in ('81299', '81298'):
        bwsdklogger.warning(
            '行情重连中..., error code=%s, info=%s. 可用on_error事件处理', code, info
        )
    else:
        bwsdklogger.warning(
            '发生错误, 调用默认的处理函数, error code=%s, info=%s.  你可以在策略里自定义on_error函数接管它. 类似于on_tick',
            code, info
        )


def err_callback(data):
    """
    遇到错误时回调, 错误代码跟错误信息的对应关系参考
    """
    if context.on_error_fun is None:
        context.on_error_fun = default_err_callback

    try:
        data_unicode = data.decode('utf8')
        sparr = data_unicode.split('|', 1)
        if len(sparr) == 1:
            code, info = "code解析不出来", sparr[0]
        else:
            code, info = sparr
        if context.on_error_fun is not None:
            context.on_error_fun(context, code, info)
    except Exception as e:
        bwsdklogger.exception("字符编码解析错误", e)
        if context.on_error_fun is not None:
            context.on_error_fun(context, "81011", data)


# 已超时触发过的eob集合, 原则上是触发过的, 即可后面在收到数据也不再次触发
already_fire_timeout_eobs = set()  # type: Set[datetime.datetime]


def timer_callback(data):
    global bars_in_waitgroup_live, already_fire_timeout_eobs
    if (context.on_bar_fun is not None) and context.has_wait_group and context.is_live_model():
        # 这里处理实时模式下wait_group=true时, on_bar超时触发
        # 比较逻辑是: 取本地时间, 然后跟相同的eob的bars里的第1个bar的 receive_local_time (接收到时的本地时间) 相比
        cur_now_s = time.time()
        must_del_keys = []
        for frequency, eob_tick_dict in six.iteritems(bars_in_waitgroup_live):
            barwaitgroupinfo = context.bar_waitgroup_frequency2Info.get(frequency, None)
            if barwaitgroupinfo is not None:
                timeout_seconds = barwaitgroupinfo.timeout_seconds
                for eob_time in list(six.iterkeys(eob_tick_dict)):
                    first_bar = eob_tick_dict[eob_time][0]  # 这个eob下的收到的第1个bar
                    delta_second = cur_now_s - first_bar['receive_local_time']  # type: float
                    if delta_second >= timeout_seconds:
                        if eob_time in already_fire_timeout_eobs:
                            bwsdklogger.debug(
                                'frequency=%s eob=%s timeout_seconds=%d, 已超时触发过, 后面又收到数据, 不进行触发',
                                frequency, eob_time
                            )
                            del eob_tick_dict[eob_time]
                            continue

                        bwsdklogger.info(
                            "frequency=%s eob=%s timeout_seconds=%d 已超时了超时秒数=%s, 触发on_bar",
                            frequency, eob_time, timeout_seconds, delta_second
                        )
                        context.on_bar_fun(context, eob_tick_dict[eob_time])
                        del eob_tick_dict[eob_time]
                        already_fire_timeout_eobs.add(eob_time)
            else:
                # 说明有些 frequency 已经退订了
                bwsdklogger.debug("frequency =%s 已全部退订", frequency)
                must_del_keys.append(frequency)

        if must_del_keys:
            for k in must_del_keys:
                del bars_in_waitgroup_live[k]
        return


def backtest_finish_callback(data):
    global pre_eob_in_backtest, bars_in_waitgroup_backtest
    # 在回测结束前, 把之前累积的bar给放出来
    if bars_in_waitgroup_backtest:
        for bs in six.itervalues(bars_in_waitgroup_backtest):
            context.on_bar_fun(context, bs)

    pre_eob_in_backtest = None
    bars_in_waitgroup_backtest = collections.defaultdict(list)
    context._temporary_now = None
    context.bar_data_set.clear()  # 清空之前的
    context.bar_data_cache.clear()
    context.tick_data_cache.clear()
    context.max_tick_data_count = 1
    context.max_bar_data_count = 1


def stop_callback(data):
    if context.on_shutdown_fun is not None:
        context.on_shutdown_fun(context)

    from bw.api import stop
    # print("!~~~~~~~~~~~!停止策略!~~~~~~~~~~~!")
    stop()


def trade_connected_callback():
    bwsdklogger.debug("连接交易服务成功")
    if context.on_trade_data_connected_fun is not None:
        context.on_trade_data_connected_fun(context)


def data_connected_callback():
    bwsdklogger.debug("连接行情服务成功")
    if context.on_market_data_connected_fun is not None:
        context.on_market_data_connected_fun(context)


def account_status_callback(data):
    if context.on_account_status_fun is not None:
        account_status = AccountStatus()
        account_status.FromFmt(data)

        account_status_dict = DictLikeAccountStatus()
        account_status_dict['account_id'] = account_status.account_id
        account_status_dict['account_name'] = account_status.account_name

        status_dict = DictLikeConnectionStatus()
        account_status_dict['status'] = status_dict
        if account_status.status is not None:
            status_dict['state'] = account_status.status.state
            error_dict = DictLikeError()
            status_dict['error'] = error_dict
            if account_status.status.error is not None:
                error_dict['code'] = account_status.status.error.code
                error_dict['type'] = account_status.status.error.type
                error_dict['info'] = account_status.status.error.info

        context.on_account_status_fun(context, account_status_dict)


def data_disconnected_callback():
    if context.on_market_data_disconnected_fun is not None:
        context.on_market_data_disconnected_fun(context)


def trade_disconnected_callback():
    if context.on_trade_data_disconnected_fun is not None:
        context.on_trade_data_disconnected_fun(context)


def callback_controller(msg_type, data):
    """
    回调任务控制器
    """
    try:
        # print('&'*20,msg_type)
        # python 3 传过来的是bytes 类型， 转成str
        if isinstance(msg_type, bytes):
            msg_type = bytes.decode(msg_type)

        # sprint('now','&'*20,msg_type)

        if msg_type == CALLBACK_TYPE_TICK:
            # print('-'*20,'tick lai le ')
            return tick_callback_new(data)

        elif msg_type == CALLBACK_TYPE_BAR:
            # print('-'*20,'bar lai le ')
            return bar_callback_new(data)

        elif msg_type == CALLBACK_TYPE_INIT:
            return init_callback(data)

        elif msg_type == CALLBACK_TYPE_SCHEDULE:
            # print('-'*20,'lai le ')
            return schedule_callback(data)

        elif msg_type == CALLBACK_TYPE_ERROR:
            return err_callback(data)

        elif msg_type == CALLBACK_TYPE_TIMER:
            # print('-'*20,'CALLBACK_TYPE_TIMER lai le ')
            return timer_callback(data)

        elif msg_type == CALLBACK_TYPE_EXECRPT:
            return excerpt_callback(data)

        elif msg_type == CALLBACK_TYPE_ORDER:
            return order_callback(data)

        elif msg_type == CALLBACK_TYPE_INDICATOR:
            # print('*'*20,'now in')
            return indicator_callback(data)

        elif msg_type == CALLBACK_TYPE_CASH:
            return cash_callback(data)

        elif msg_type == CALLBACK_TYPE_POSITION:
            return position_callback(data)

        elif msg_type == CALLBACK_TYPE_PARAMETERS:
            return parameters_callback(data)

        elif msg_type == CALLBACK_TYPE_BACKTEST_FINISH:
            return backtest_finish_callback(data)

        elif msg_type == CALLBACK_TYPE_STOP:
            return stop_callback(data)

        elif msg_type == CALLBACK_TYPE_TRADE_CONNECTED:
            return trade_connected_callback()

        elif msg_type == CALLBACK_TYPE_TRADE_DISCONNECTED:
            return trade_disconnected_callback()

        elif msg_type == CALLBACK_TYPE_DATA_CONNECTED:
            return data_connected_callback()

        elif msg_type == CALLBACK_TYPE_DATA_DISCONNECTED:
            return data_disconnected_callback()

        elif msg_type == CALLBACK_TYPE_ACCOUNTSTATUS:
            return account_status_callback(data)

        bwsdklogger.warn("没有处理消息:%s的处理函数", msg_type)

    except SystemExit:
        bwsdklogger.debug("callback_controller^^--------------SystemExit--------------^^")
        from bw.api import stop
        stop()

    except BaseException as e:
        bwsdklogger.exception("^^--------------遇到exception--------------^^")
        from bw.api import stop
        stop()



# coding=utf-8
from __future__ import unicode_literals, print_function, absolute_import

import six
from typing import Text, List, Dict, NoReturn, Any, Union

from bw.csdk.cpp_sdk import py_bwi_place_order, py_bwi_get_unfinished_orders, py_bwi_get_orders, py_bwi_get_positions,\
    py_bwi_cancel_all_orders, py_bwi_close_all_positions, py_bwi_cancel_order, py_bwi_get_execution_reports,\
        py_bwi_get_sector,COrder
    
from bw.enum import OrderQualifier_Unknown, OrderDuration_Unknown
from bw.model.storage import context
from bw.yy.data import *

from bw.yy.account import Order, OrderStyle_Volume, OrderStyle_Value, OrderStyle_Percent, \
    OrderStyle_TargetVolume, OrderStyle_TargetValue, OrderStyle_TargetPercent, ExecRpts,PositionSide_Unknown, \
PositionSide_Long,PositionSide_Short,OrderSide_Unknown ,OrderSide_Buy ,OrderSide_Sell



from bw.utils import load_to_list, datetime2timestamp


def _inner_place_order(o):
    # type: (Order) ->List[Dict[Text, Any]]
    """
    下单并返回order的信息. 同步调用, 在下单返回的order,错误返回空的list
    """
    # # 在回测模式且wait_group=True时, 设置这个created_at, 也就是通知c底层要根据这个时间设置price
    if context.is_backtest_model() and context.has_wait_group:
        o.created_at =  datetime2timestamp( context.now ) *1000  # millisec --->cppsdk cover
    # print(o)
    
    result = py_bwi_place_order(o)
    
    return result


def order_volume(symbol, volume, side, order_type, position_effect,
                 price=0, order_duration=OrderDuration_Unknown, order_qualifier=OrderQualifier_Unknown, account=''):
    # type: (Text, float, int, int, int, float, int, int, Text) ->List[Dict[Text, Any]]
    """
    按指定量委托
    """
    order_style = OrderStyle_Volume
    # print('order_volume start')
    account_id = get_account_id(account)
    # print('account=',account,'account_id=',account_id)

    o = COrder()
    o.symbol = symbol
    o.volume = (volume)
    o.price = price
    o.side = side
    o.order_type = order_type
    o.position_effect = position_effect
    o.order_style = order_style
    o.order_qualifier = order_qualifier
    o.order_duration = order_duration
    o.account_id = account_id

    return _inner_place_order(o)


def order_value(symbol, value, side, order_type, position_effect,
                price=0, order_duration=OrderDuration_Unknown, 
                order_qualifier=OrderQualifier_Unknown, account=''):
    # type:(Text, float, int, int, int, float, int, int, Text) ->List[Dict[Text, Any]]
    """
    按指定价值委托
    """
    order_style = OrderStyle_Value
    account_id = get_account_id(account)

    o = COrder()
    o.symbol = symbol
    o.value = value
    o.price = price
    o.side = side
    o.order_type = order_type
    o.position_effect = position_effect
    o.order_style = order_style
    o.order_qualifier = order_qualifier
    o.order_duration = order_duration
    o.account_id = account_id

    return _inner_place_order(o)


def order_percent(symbol, percent, side, order_type, position_effect,
                  price=0, order_duration=OrderDuration_Unknown, order_qualifier=OrderQualifier_Unknown, account=''):
    # type: (Text, float, int, int, int, float, int, int, Text)->List[Dict[Text, Any]]
    """
    按指定比例委托
    """
    order_style = OrderStyle_Percent
    account_id = get_account_id(account)

    o = COrder()
    o.symbol = symbol
    o.percent = percent
    o.price = price
    o.side = side
    o.order_type = order_type
    o.position_effect = position_effect
    o.order_qualifier = order_qualifier
    o.order_duration = order_duration
    o.order_style = order_style
    o.account_id = account_id

    return _inner_place_order(o)


def order_target_volume(symbol, volume, position_side, order_type, price=0, order_duration=OrderDuration_Unknown,
                        order_qualifier=OrderQualifier_Unknown, account=''):
    # type: (Text, float, int, int, float, int, int, Text) ->List[Dict[Text, Any]]
    """
    调仓到目标持仓量
    """
    order_style = OrderStyle_TargetVolume

    account_id = get_account_id(account)
    o = COrder()
    o.symbol = symbol
    o.target_volume = volume
    o.price = price
    o.position_side = position_side
    o.order_type = order_type
    o.order_qualifier = order_qualifier
    o.order_duration = order_duration
    o.order_style = order_style
    o.account_id = account_id
    if position_side == PositionSide_Unknown:
        raise Exception('持仓类型不正确!')
    o.side = OrderSide_Buy if position_side==PositionSide_Long else OrderSide_Sell


    return _inner_place_order(o)


def order_target_value(symbol, value, position_side, order_type, price=0,
                       order_duration=OrderDuration_Unknown,
                        order_qualifier=OrderQualifier_Unknown, account=''):
    # type: (Text, float, int, int, float, int, int, Text) ->List[Dict[Text, Any]]
    """
    调仓到目标持仓额
    """
    order_style = OrderStyle_TargetValue
    account_id = get_account_id(account)

    o = COrder()
    o.symbol = symbol
    o.target_value = value
    o.price = price
    o.position_side = position_side
    o.order_type = order_type
    o.order_qualifier = order_qualifier
    o.order_duration = order_duration
    o.order_style = order_style
    o.account_id = account_id
    if position_side == PositionSide_Unknown:
        raise Exception('持仓类型不正确!')
    o.side = OrderSide_Buy if position_side==PositionSide_Long else OrderSide_Sell
    # o.side = position_side== PositionSide.

    return _inner_place_order(o)


def order_target_percent(symbol, percent, position_side, order_type, price=0, order_duration=OrderDuration_Unknown,
                         order_qualifier=OrderQualifier_Unknown, account=''):
    # type: (Text, float, int, int, float, int, int, Text) ->List[Dict[Text, Any]]
    """
    调仓到目标持仓比例
    """
    order_style = OrderStyle_TargetPercent
    account_id = get_account_id(account)

    o = COrder()
    o.symbol = symbol
    o.target_percent = percent
    o.price = price
    o.position_side = position_side
    o.order_type = order_type
    o.order_qualifier = order_qualifier
    o.order_duration = order_duration
    o.order_style = order_style
    o.account_id = account_id
    if position_side == PositionSide_Unknown:
        raise Exception('持仓类型不正确!')
    o.side = OrderSide_Buy if position_side==PositionSide_Long else OrderSide_Sell

    return _inner_place_order(o)


def get_unfinished_orders():
    # type: ()->List[Dict[Text, Any]]
    """
    查询所有未结委托
    """
    unfinished_orders = []
    for account in six.itervalues(context.accounts):
        # print('-*'*50,account.id)
        result = py_bwi_get_unfinished_orders(account.id)
        if result:
            unfinished_orders.extend(result)

    return unfinished_orders


def get_orders():
    # type: () ->List[Dict[Text, Any]]
    """
    查询日内全部委托
    """
    all_orders = []
    for account in six.itervalues(context.accounts):
        result= py_bwi_get_orders(account.id)
        # print('*---'*20,len(result),account.id)
        if result:
            all_orders.extend(result)
    return all_orders

def get_positions():
    # type: () ->List[Dict[Text, Any]]
    """
    查询日内全部委托
    """
    all_pos = []
    for account in six.itervalues(context.accounts):
        result= py_bwi_get_positions(account.id)
        # print('*---'*20,len(result),account.id)
        if result:
            all_pos.extend(result)
    return all_pos

def order_cancel_all():
    # type: () ->NoReturn
    """
    撤销所有委托
    """
    py_bwi_cancel_all_orders()


def order_close_all():
    # type: () ->List[Dict[Text, Any]]
    """
    平当前所有可平持仓
    """
    py_bwi_close_all_positions()
    #return oder list
    return None


def order_cancel(wait_cancel_orders):
    # type: (Union[Dict[Text,Any], List[Dict[Text, Any]]]) ->NoReturn
    """
    撤销委托. 传入单个字典. 或者list字典. 每个字典包含key: cl_ord_id, account_id
    """
    wait_cancel_orders = load_to_list(wait_cancel_orders)

    py_bwi_cancel_order(wait_cancel_orders)


# def order_batch(order_infos,account=''):
#     """
#     批量委托接口
#     """

#     py_bwi_place_order()
#     res = Orders()
#     return res


def get_account_id(name_or_id):
    # type: (Text) ->Text
    for one in six.itervalues(context.accounts):
        if one.match(name_or_id):
            return one.id
    # print('not matched ',name_or_id)
    # 都没有匹配上, 等着后端去拒绝
    if  name_or_id.strip().__len__() == 0:
        return "DEFAULT"
    return name_or_id


def get_execution_reports():
    # type: () -> List[Dict[Text, Any]]
    """
    返回执行回报
    """
    reports = []
    for account in six.itervalues(context.accounts):
        req = account.id
        result = py_bwi_get_execution_reports(req)
        if result:
            reports.extend(result)
            
    return reports
